package me.unurled.sacredrealms.sr.components.cutscene;

import static me.unurled.sacredrealms.sr.utils.Items.cancelItem;
import static me.unurled.srcore.utils.Component.textComp;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import me.unurled.sacredrealms.sr.SR;
import me.unurled.sacredrealms.sr.components.cutscene.tasks.TickTracker;
import me.unurled.srcore.api.Manager;
import net.kyori.adventure.text.TextComponent;
import org.bukkit.Material;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.PlayerInventory;
import org.bukkit.inventory.meta.ItemMeta;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public class CutsceneManager extends Manager {

  private static final TextComponent ALREADY_STARTED =
      textComp("<red>You are already creating a cutscene.");

  private List<Cutscene> cutscenes;
  private Map<UUID, Cutscene> cutsceneBeingCreated;
  private Map<UUID, PlayerInventory> savedInventories;

  @Override
  public void load() {
    super.load();
    SR.getPlugin()
        .getServer()
        .getPluginManager()
        .registerEvents(new CreationListener(), SR.getPlugin());
    TickTracker.startTracking();
    cutscenes = new ArrayList<>();
    cutsceneBeingCreated = new HashMap<>();
    savedInventories = new HashMap<>();
  }

  @Override
  public void unload() {
    super.unload();
    cutscenes.clear();
    cutsceneBeingCreated.clear();
    savedInventories.clear();
  }

  @Nullable
  public Cutscene getCutsceneBeingCreated(@NotNull UUID uuid) {
    return cutsceneBeingCreated.get(uuid);
  }

  public void startCreatingCutscene(@NotNull Player p, String name) {
    if (cutsceneBeingCreated.containsKey(p.getUniqueId())) {
      p.sendMessage(ALREADY_STARTED);
      return;
    }
    cutsceneBeingCreated.put(p.getUniqueId(), new Cutscene(name));

    // process to create cutscene
    savedInventories.put(p.getUniqueId(), p.getInventory());
    p.getInventory().clear();
    p.sendMessage(
        textComp("<green>You are now creating a cutscene. Place the <gold>corner blocks</gold>."));
    ItemStack cancel = cancelItem();
    ItemStack finish = new ItemStack(Material.EMERALD_BLOCK);
    ItemMeta meta = finish.getItemMeta();
    meta.displayName(textComp("<b><green>Finish</green></b>"));
    finish.setItemMeta(meta);

    ItemStack pos1 = new ItemStack(Material.RED_WOOL);
    meta = pos1.getItemMeta();
    meta.displayName(textComp("<b><red>Pos1</red></b>"));
    pos1.setItemMeta(meta);

    ItemStack pos2 = new ItemStack(Material.BLUE_WOOL);
    meta = pos2.getItemMeta();
    meta.displayName(textComp("<b><blue>Pos2</blue></b>"));
    pos2.setItemMeta(meta);

    p.getInventory().setItem(8, cancel);
    p.getInventory().setItem(0, finish);
    p.getInventory().setItem(3, pos1);
    p.getInventory().setItem(5, pos2);

    p.updateInventory();
  }

  public void finishCreatingCutscene(@NotNull Player p) {
    if (!cutsceneBeingCreated.containsKey(p.getUniqueId())) {
      p.sendMessage(textComp("<red>You are not creating a cutscene."));
      return;
    }
    Cutscene cutscene = cutsceneBeingCreated.get(p.getUniqueId());

    cutscenes.add(cutscene);
    p.getInventory().clear();
    p.sendMessage(textComp("<green>Cutscene created."));
    cutsceneBeingCreated.remove(p.getUniqueId());
    PlayerInventory inv = savedInventories.get(p.getUniqueId());
    for (int i = 0; i < inv.getSize(); i++) {
      p.getInventory().setItem(i, inv.getItem(i));
    }
    p.updateInventory();
    savedInventories.remove(p.getUniqueId());
  }

  public void cancelCreatingCutscene(@NotNull Player p) {
    if (!cutsceneBeingCreated.containsKey(p.getUniqueId())) {
      p.sendMessage(textComp("<red>You are not creating a cutscene."));
      return;
    }

    p.getInventory().clear();
    p.sendMessage(textComp("<green>Canceled creating cutscene."));
    cutsceneBeingCreated.remove(p.getUniqueId());
    PlayerInventory inv = savedInventories.get(p.getUniqueId());
    for (int i = 0; i < inv.getSize(); i++) {
      p.getInventory().setItem(i, inv.getItem(i));
    }
    p.updateInventory();
    savedInventories.remove(p.getUniqueId());
  }

  public List<Cutscene> cutscenes() {
    return cutscenes;
  }

  @Nullable
  public Cutscene findByUUID(UUID uuid) {
    return cutscenes.stream().filter(c -> c.id().equals(uuid)).findFirst().orElse(null);
  }
}
