package me.unurled.sacredrealms.sr.components.cutscene;

import static me.unurled.srcore.utils.Component.textComp;

import java.util.ArrayList;
import java.util.List;
import me.unurled.sacredrealms.sr.components.cutscene.recording.Recordable;
import me.unurled.sacredrealms.sr.components.cutscene.recording.Recording;
import me.unurled.sacredrealms.sr.components.cutscene.tasks.TickTracker;
import me.unurled.srcore.api.Manager;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public class RecordingManager extends Manager {
  private List<Recording> recordings; // existing recordings
  private List<Recording> activeRecordings; // currently recording

  @Override
  public void load() {
    super.load();
    recordings = new ArrayList<>();
    activeRecordings = new ArrayList<>();
  }

  @Override
  public void unload() {
    super.unload();
    recordings.clear();
    activeRecordings.clear();
  }

  @Nullable
  public Recording getPlayerActiveRecording(Player player) {
    return activeRecordings.stream()
        .filter(r -> r.recordingPlayer().equals(player))
        .findFirst()
        .orElse(null);
  }

  public void startRecording(Cutscene cutscene, Player player) {
    Recording recording = new Recording(cutscene, player);
    player.sendMessage(textComp("<green>Recording started."));
    activeRecordings.add(recording);
  }

  public void stopRecording(Player player) {
    Recording recording = getPlayerActiveRecording(player);
    if (recording == null) {
      player.sendMessage(textComp("<red>You are not recording a cutscene."));
      return;
    }
    activeRecordings.remove(recording);
    recording.stopRecording();
    recordings.add(recording);
    player.sendMessage(textComp("<green>Recording stopped."));
  }

  public void addRecordable(@NotNull Recording recording, @NotNull Recordable recordable) {
    long tick = TickTracker.currentTick();

    // normalize tick
    tick = tick - recording.startTick();

    recordable.setTick(tick);

    if (recording.recordables().containsKey(tick)) {
      recording.recordables().get(tick).add(recordable);
    } else {
      List<Recordable> recordables = new ArrayList<>();
      recordables.add(recordable);
      recording.recordables().put(tick, recordables);
    }
  }
}
