package me.unurled.sacredrealms.sr.components.cutscene.recording;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Queue;
import java.util.UUID;
import java.util.concurrent.ConcurrentLinkedQueue;
import me.unurled.sacredrealms.sr.SR;
import me.unurled.sacredrealms.sr.components.cutscene.Cutscene;
import me.unurled.sacredrealms.sr.components.cutscene.tasks.ScanEntityTracker;
import me.unurled.sacredrealms.sr.components.cutscene.tasks.TickTracker;
import me.unurled.sacredrealms.sr.components.cutscene.tasks.TrackLocationTask;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.bukkit.scheduler.BukkitTask;

public class Recording {
  // recordables
  private final Map<Long, List<Recordable>> recordables = new HashMap<>();
  private final Queue<UUID> recordableEntities = new ConcurrentLinkedQueue<>();
  private final BukkitTask trackLocationTask;
  private final BukkitTask scanEntitiesTask;
  private Cutscene cutscene;
  private Player recordingPlayer;
  private long startTime;
  private long endTime;
  private long startTick;
  private long endTick;

  public Recording(Cutscene cutscene, Player recordingPlayer) {
    this.cutscene = cutscene;
    this.recordingPlayer = recordingPlayer;

    this.startTime = System.currentTimeMillis();
    this.startTick = 1; // TODO: get tick
    this.endTick = 1; // TODO: get tick

    this.scanEntitiesTask =
        Bukkit.getScheduler().runTaskTimer(SR.getPlugin(), new ScanEntityTracker(this), 0L, 20L);
    this.trackLocationTask =
        Bukkit.getScheduler().runTaskTimer(SR.getPlugin(), new TrackLocationTask(this), 0L, 1L);
  }

  public Map<Long, List<Recordable>> recordables() {
    return recordables;
  }

  public Cutscene cutscene() {
    return cutscene;
  }

  public Queue<UUID> recordableEntities() {
    return recordableEntities;
  }

  public Recording setCutscene(Cutscene cutscene) {
    this.cutscene = cutscene;
    return this;
  }

  public Player recordingPlayer() {
    return recordingPlayer;
  }

  public Recording setRecordingPlayer(Player recordingPlayer) {
    this.recordingPlayer = recordingPlayer;
    return this;
  }

  public long startTime() {
    return startTime;
  }

  public Recording setStartTime(long startTime) {
    this.startTime = startTime;
    return this;
  }

  public long endTime() {
    return endTime;
  }

  public Recording setEndTime(long endTime) {
    this.endTime = endTime;
    return this;
  }

  public long startTick() {
    return startTick;
  }

  public Recording setStartTick(long startTick) {
    this.startTick = startTick;
    return this;
  }

  public void stopRecording() {
    this.endTime = System.currentTimeMillis();
    this.endTick = TickTracker.currentTick();
    scanEntitiesTask.cancel();
    trackLocationTask.cancel();
  }

  public long tickDuration() {
    return endTick - startTick;
  }

  public boolean isEntityBeingTracked(UUID uuid) {
    synchronized (recordableEntities) {
      return recordableEntities.contains(uuid);
    }
  }

  public boolean addEntityIfNotBeingTracked(UUID uuid) {
    synchronized (recordableEntities) {
      if (recordableEntities.contains(uuid)) {
        return false;
      }
      recordableEntities.add(uuid);
      return true;
    }
  }
}
