package me.unurled.sacredrealms.sr.components.cutscene.recording.recordables;

import com.github.retrooper.packetevents.protocol.entity.type.EntityTypes;
import com.github.retrooper.packetevents.protocol.player.User;
import com.github.retrooper.packetevents.protocol.player.UserProfile;
import com.github.retrooper.packetevents.wrapper.play.server.WrapperPlayServerPlayerInfoUpdate;
import com.github.retrooper.packetevents.wrapper.play.server.WrapperPlayServerSpawnEntity;
import com.github.retrooper.packetevents.wrapper.play.server.WrapperPlayServerSpawnPlayer;
import io.github.retrooper.packetevents.util.SpigotConversionUtil;
import java.lang.reflect.Field;
import java.util.EnumSet;
import java.util.UUID;
import java.util.concurrent.atomic.AtomicInteger;
import me.unurled.sacredrealms.sr.components.cutscene.Replay;
import me.unurled.sacredrealms.sr.components.cutscene.recording.Recordable;
import net.kyori.adventure.text.Component;
import org.bukkit.GameMode;
import org.bukkit.Location;
import org.bukkit.entity.Entity;
import org.bukkit.entity.EntityType;
import org.jetbrains.annotations.NotNull;

public class SpawnEntityRecordable extends Recordable {

  private EntityType entityType;
  private Location location;
  private UUID entityId;
  private Component name;

  public SpawnEntityRecordable() {}

  public SpawnEntityRecordable(@NotNull Entity e) {
    this.entityType = e.getType();
    this.location = e.getLocation();
    this.entityId = e.getUniqueId();
    this.name = e.name();
  }

  public SpawnEntityRecordable(
      EntityType entityType, Location location, UUID entityId, Component name) {
    this.entityType = entityType;
    this.location = location;
    this.entityId = entityId;
    this.name = name;
  }

  @Override
  public void replay(Replay replay, User user) throws Exception { // TODO: check everything

    Class<?> entityClass = Class.forName("net.minecraft.world.entity.Entity");
    Field field = entityClass.getDeclaredField("ENTITY_COUNTER");
    field.setAccessible(true);
    AtomicInteger ENTITY_COUNTER = (AtomicInteger) field.get(null);
    int id = ENTITY_COUNTER.incrementAndGet();

    if (entityType == EntityType.PLAYER) {
      WrapperPlayServerPlayerInfoUpdate.PlayerInfo data =
          new WrapperPlayServerPlayerInfoUpdate.PlayerInfo(
              new UserProfile(entityId, name.toString()),
              false,
              0,
              SpigotConversionUtil.fromBukkitGameMode(GameMode.SURVIVAL),
              name,
              null);

      user.sendPacket(
          new WrapperPlayServerPlayerInfoUpdate(
              EnumSet.of(WrapperPlayServerPlayerInfoUpdate.Action.ADD_PLAYER), data));
      user.sendPacket(
          new WrapperPlayServerSpawnPlayer(
              id, entityId, SpigotConversionUtil.fromBukkitLocation(location)));

      replay.spawnedEntities().put(entityId, id);
    } else {
      com.github.retrooper.packetevents.protocol.entity.type.EntityType entityType1 =
          EntityTypes.getByName(entityType.getKey().toString());

      WrapperPlayServerSpawnEntity spawnEntityPacket =
          new WrapperPlayServerSpawnEntity(
              id,
              UUID.randomUUID(),
              entityType1,
              SpigotConversionUtil.fromBukkitLocation(location),
              0,
              0,
              null);

      user.sendPacket(spawnEntityPacket);
      replay.spawnedEntities().put(entityId, id);
    }
  }

  public EntityType entityType() {
    return entityType;
  }

  public SpawnEntityRecordable setEntityType(EntityType entityType) {
    this.entityType = entityType;
    return this;
  }

  public Location location() {
    return location;
  }

  public SpawnEntityRecordable setLocation(Location location) {
    this.location = location;
    return this;
  }

  public UUID entityId() {
    return entityId;
  }

  public SpawnEntityRecordable setEntityId(UUID entityId) {
    this.entityId = entityId;
    return this;
  }

  public Component name() {
    return name;
  }

  public SpawnEntityRecordable setName(Component name) {
    this.name = name;
    return this;
  }
}
