package me.unurled.sacredrealms.sr.components.cutscene.tasks;

import java.util.ArrayList;
import java.util.List;
import me.unurled.sacredrealms.sr.components.cutscene.Cutscene;
import me.unurled.sacredrealms.sr.components.cutscene.RecordingManager;
import me.unurled.sacredrealms.sr.components.cutscene.recording.Recording;
import me.unurled.sacredrealms.sr.components.cutscene.recording.recordables.SpawnEntityRecordable;
import me.unurled.srcore.api.Manager;
import org.bukkit.Chunk;
import org.bukkit.Location;

public class ScanEntityTracker implements Runnable {

  private final Recording recording;

  public ScanEntityTracker(Recording recording) {
    this.recording = recording;
  }

  @Override
  public void run() {
    Cutscene cutscene = recording.cutscene();
    Location pos1 = cutscene.pos1();
    Location pos2 = cutscene.pos2();
    int chunkZ = pos1.getBlockZ() >> 4;
    int chunkX = pos1.getBlockX() >> 4;
    int chunkX2 = pos2.getBlockX() >> 4;
    int chunkZ2 = pos2.getBlockZ() >> 4;

    var loadedChunks = pos1.getWorld().getLoadedChunks();
    List<Chunk> chunksInRegion = new ArrayList<>();
    for (Chunk chunk : loadedChunks) {
      if (chunk.getX() >= chunkX
          && chunk.getX() <= chunkX2
          && chunk.getZ() >= chunkZ
          && chunk.getZ() <= chunkZ2) {
        chunksInRegion.add(chunk);
      }
    }

    for (Chunk chunk : chunksInRegion) {
      var entities = chunk.getEntities();
      for (var entity : entities) {
        if (!cutscene.isInRegion(entity.getLocation())) {
          continue;
        }

        if (recording.addEntityIfNotBeingTracked(entity.getUniqueId())) {
          // TODO: spawn entity
          SpawnEntityRecordable recordable = new SpawnEntityRecordable(entity);
          Manager.getInstance(RecordingManager.class).addRecordable(recording, recordable);
        }
      }
    }
  }
}
