package me.unurled.sacredrealms.sr.components.pointer;

import com.github.retrooper.packetevents.PacketEvents;
import com.github.retrooper.packetevents.protocol.entity.data.EntityData;
import com.github.retrooper.packetevents.protocol.entity.data.EntityDataTypes;
import com.github.retrooper.packetevents.protocol.player.User;
import com.github.retrooper.packetevents.wrapper.play.server.WrapperPlayServerEntityMetadata;
import com.github.retrooper.packetevents.wrapper.play.server.WrapperPlayServerSpawnEntity;
import io.github.retrooper.packetevents.util.SpigotConversionUtil;
import java.util.ArrayList;
import java.util.List;
import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.block.BlockState;
import org.bukkit.entity.EntityType;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;
import org.joml.Vector3f;

// TODO: test this
public class DisplayForPlayer implements Runnable {

  private static final int RENDER_DISTANCE_BLOCKS = 128;

  private final Pointer pointer;
  private final Player player;
  private final User user;

  public DisplayForPlayer(Pointer pointer, Player player) {
    this.pointer = pointer;
    this.player = player;
    this.user = PacketEvents.getAPI().getPlayerManager().getUser(player);
  }

  public static Location getLocationInRange(@NotNull Player player, Location targetLocation) {
    Location playerLocation = player.getLocation();

    // Calculate the distance between player and target location
    double distance = playerLocation.distance(targetLocation);

    // If the location is within the render distance, return the original location
    if (distance <= RENDER_DISTANCE_BLOCKS) {
      return targetLocation;
    }

    // If the location is out of range, calculate a new location within the render distance
    // Calculate the direction vector from the player to the target
    Location direction =
        targetLocation
            .clone()
            .subtract(playerLocation)
            .toVector()
            .normalize()
            .toLocation(player.getWorld());

    // Create a new location within the render distance pointing towards the target location
    return playerLocation.clone().add(direction.multiply(RENDER_DISTANCE_BLOCKS));
  }

  @Override
  public void run() {
    // get difference between player loc and pointer loc, if it is too large then set pointer
    // blockdisplay's location to in range of player

    Location location = getLocationInRange(player, pointer.location());

    WrapperPlayServerSpawnEntity packet =
        new WrapperPlayServerSpawnEntity(
            pointer.entityId(),
            pointer.entityUUID(),
            SpigotConversionUtil.fromBukkitEntityType(EntityType.BLOCK_DISPLAY),
            SpigotConversionUtil.fromBukkitLocation(location),
            0,
            0,
            null);

    user.sendPacket(packet);

    List<EntityData> list = new ArrayList<>();

    BlockState ironBar =
        player
            .getWorld()
            .getBlockAt(new Location(player.getWorld(), 0, 0, 0))
            .getState()
            .getBlockData()
            .createBlockState();
    ironBar.setType(Material.IRON_BARS);

    // make it iron bars
    EntityData data = new EntityData(23, EntityDataTypes.BLOCK_STATE, ironBar);
    list.add(data);

    // scale
    EntityData data2 = new EntityData(12, EntityDataTypes.VECTOR3F, new Vector3f(2, 150, 2));
    list.add(data2);

    // glowing
    EntityData glowing = new EntityData(0, EntityDataTypes.BYTE, (byte) 0x40);
    list.add(glowing);

    WrapperPlayServerEntityMetadata packet1 =
        new WrapperPlayServerEntityMetadata(pointer.entityId(), list);

    user.sendPacket(packet1);
  }
}
