package me.unurled.sacredrealms.sr.utils;

import static me.unurled.sacredrealms.sr.utils.Logger.error;
import static me.unurled.sacredrealms.sr.utils.Logger.log;
import static org.bukkit.Bukkit.getServer;

import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.World;
import org.bukkit.entity.Entity;
import org.bukkit.entity.EntityType;
import org.bukkit.entity.Slime;
import org.bukkit.event.entity.CreatureSpawnEvent.SpawnReason;
import org.bukkit.scoreboard.Scoreboard;
import org.bukkit.scoreboard.Team;
import org.jetbrains.annotations.NotNull;

public class RegionUtil {

  private static final String REGIONSELECTOR = "srselector";

  public static void killSelectorsWithTag(String inputTag) {
    for (World world : getServer().getWorlds()) {
      for (Entity entity : world.getEntities()) {
        for (String tag : entity.getScoreboardTags()) {
          if (tag.equals(REGIONSELECTOR + "-" + inputTag)) {
            entity.remove();
            break;
          }
        }
      }
    }
  }

  public static void killAllSelectors() {
    for (World world : getServer().getWorlds()) {
      for (Entity entity : world.getEntities()) {
        for (String tag : entity.getScoreboardTags()) {
          if (tag.startsWith(REGIONSELECTOR + "-")) {
            entity.remove();
            break;
          }
        }
      }
    }
  }

  public static void drawSelector(
      @NotNull Region region, World world, String id, NamedTextColor glowColor) {
    Location loc1 = region.corner1();
    Location loc2 = region.corner2();
    if (loc1.getWorld() == null || !loc1.getWorld().equals(loc2.getWorld())) {
      return;
    }
    int minx = Math.min(loc1.getBlockX(), loc2.getBlockX());
    int miny = Math.min(loc1.getBlockY(), loc2.getBlockY());
    int minz = Math.min(loc1.getBlockZ(), loc2.getBlockZ());
    int maxx = Math.max(loc1.getBlockX(), loc2.getBlockX());
    int maxy = Math.max(loc1.getBlockY(), loc2.getBlockY());
    int maxz = Math.max(loc1.getBlockZ(), loc2.getBlockZ());

    Scoreboard board = Bukkit.getScoreboardManager().getMainScoreboard();

    int highestNumber = 0;

    for (Team team : board.getTeams()) {
      if (team.getName().startsWith(REGIONSELECTOR + "+")) {
        String number = team.getName().substring(11);
        try {
          int tempNumber = Integer.parseInt(number);
          if (tempNumber >= highestNumber) {
            highestNumber = tempNumber;
          }
        } catch (NumberFormatException e) {
          error(e.getMessage());
        }
      }
    }

    log(REGIONSELECTOR + "+" + (highestNumber + 1));

    Team selectorTeam = board.registerNewTeam(REGIONSELECTOR + "+" + (highestNumber + 1));
    if (glowColor != null) {
      selectorTeam.color(glowColor);
    }
    selectorTeam.setOption(Team.Option.COLLISION_RULE, Team.OptionStatus.NEVER);

    for (double x = minx; x <= maxx; x++) {
      makeSlime(id, world, selectorTeam, x, miny, minz);
      makeSlime(id, world, selectorTeam, x, maxy, minz);
      makeSlime(id, world, selectorTeam, x, miny, maxz);
      makeSlime(id, world, selectorTeam, x, maxy, maxz);
    }

    for (double y = miny; y <= maxy; y++) {
      makeSlime(id, world, selectorTeam, minx, y, minz);
      makeSlime(id, world, selectorTeam, maxx, y, minz);
      makeSlime(id, world, selectorTeam, minx, y, maxz);
      makeSlime(id, world, selectorTeam, maxx, y, maxz);
    }

    for (double z = minz; z <= maxz; z++) {
      makeSlime(id, world, selectorTeam, minx, miny, z);
      makeSlime(id, world, selectorTeam, maxx, miny, z);
      makeSlime(id, world, selectorTeam, minx, maxy, z);
      makeSlime(id, world, selectorTeam, maxx, maxy, z);
    }
  }

  private static void makeSlime(
      String id, @NotNull World world, @NotNull Team selectorTeam, double x, double y, double z) {
    Slime slime =
        (Slime)
            world.spawnEntity(
                new Location(world, x + 0.5, y, z + 0.5), EntityType.SLIME, SpawnReason.CUSTOM);
    slime.setSize(2);
    slime.setAI(false);
    slime.setGravity(false);
    slime.setCollidable(false);
    slime.setSilent(true);
    slime.setCanPickupItems(false);
    slime.setGlowing(true);
    slime.setInvulnerable(true);
    slime.setInvisible(true);
    slime.addScoreboardTag(REGIONSELECTOR + "-" + id);
    slime.setPersistent(true);
    selectorTeam.addEntry(String.valueOf(slime.getUniqueId()));
  }

  public void removeTempTeams() {
    Scoreboard board = Bukkit.getScoreboardManager().getMainScoreboard();
    for (Team team : board.getTeams()) {
      if (team.getName().startsWith(REGIONSELECTOR + "+")) {
        team.unregister();
      }
    }
  }
}
